--
--
-- BuyableStrawChopper
-- specialization for Buyable Straw Chopper
--
-- author: Burner
-- date: 21.08.2013
--
--

BuyableStrawChopper = {};

function BuyableStrawChopper.initSpecialization()
	Vehicle.registerJointType("strawChopper");
end;

function BuyableStrawChopper.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Attachable, specializations);
end;

function BuyableStrawChopper:load(xmlFile)
	self.objectInRange = BuyableStrawChopper.objectInRange;
	self.activateChopper = SpecializationUtil.callSpecializationsFunction("activateChopper");
	self.chopperActive = false;
	 
	self.chopperOptionsNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.strawChopperActionPoint#index"));

	self.combineExtraObjects = {};
	local i = 0;
	while true do
		local key = string.format("vehicle.combineExtraObjects.combineExtraObject(%d)", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;
		local showOnAttach = Utils.getNoNil(getXMLBool(xmlFile, key.."#showOnAttach"), false);
		local index = getXMLString(xmlFile, key.."#index");
		if index ~= nil then
			table.insert(self.combineExtraObjects, {index=index, showOnAttach=showOnAttach});
		end;
		i = i +1;
	end;

	self.chopperParticleSystems = {};
	local i = 0;
	while true do
		local key = string.format("vehicle.chopperParticleSystems.chopperParticleSystem(%d)", i);
		local t = getXMLString(xmlFile, key .. "#type");
		if t == nil then
			break;
		end;
		local desc = FruitUtil.fruitTypes[t];
		if desc ~= nil then
			local currentPS = {};

			local particleNode = Utils.loadParticleSystem(xmlFile, currentPS, key, self.components, false, "$data/vehicles/particleSystems/threshingChopperParticleSystem.i3d", self.baseDirectory);
			self.chopperParticleSystems[desc.index] = currentPS;
		end;
		i = i + 1;
	end;
	
	self.threshingUVScrollParts = {};
	local i = 0;
	while true do
		local key = string.format("vehicle.threshingUVScrollParts.threshingUVScrollPart(%d)", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;
		local node = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
		local speed = Utils.getVectorNFromString(getXMLString(xmlFile, key.."#speed"), 2);
		if node ~= nil and speed then
			table.insert(self.threshingUVScrollParts, {node=node, speed=speed});
		end;
		i = i +1;
	end;
	
	self.chopperParticleSystemsBackup = {};
	self.av = {};
	
	self.waitForAttach = false;
	self.collisionMaskBackup = 0;
end;

function BuyableStrawChopper:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'chopperActivated="'.. tostring(self.chopperActive) ..'"';
	return attributes, nil;
end;

function BuyableStrawChopper:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicles then
		local chopperActivated = Utils.getNoNil(getXMLBool(xmlFile, key.."#chopperActivated"), false);
		if chopperActivated then
			self.waitForAttach = true;
		end;
	end;
	return BaseMission.VEHICLE_LOAD_OK;
end;

function BuyableStrawChopper:mouseEvent(posX, posY, isDown, isUp, button)
end;

function BuyableStrawChopper:delete()
	for _,v in pairs(self.chopperParticleSystemsBackup) do
		if v ~= nil then
			Utils.deleteParticleSystem(v);
		end;
	end;
	for _,v in pairs(self.chopperParticleSystems) do
		if v ~= nil then
			Utils.deleteParticleSystem(v);
		end;
	end;
end;

function BuyableStrawChopper:keyEvent(unicode, sym, modifier, isDown)
end;

function BuyableStrawChopper:readStream(streamId, connection)
	self:activateChopper(streamReadBool(streamId), true);
end;

function BuyableStrawChopper:writeStream(streamId, connection)
	streamWriteBool(streamId, self.chopperActive);
end;

function BuyableStrawChopper:update(dt)
	if self.waitForAttach and self.attacherVehicle ~= nil then
		self:activateChopper(true);
		self.waitForAttach = false;
	end;

	if self.isClient then
		if self:objectInRange(self.chopperOptionsNode) and self.attacherVehicle ~= nil then
			if InputBinding.hasEvent(InputBinding.BizonGigant_ActivateChopper) then
				self:activateChopper(not self.chopperActive);
			end;
			if self.chopperActive then
				g_currentMission:addHelpButtonText(g_i18n:getText("BizonGigant_DeactivateChopper"), InputBinding.BizonGigant_ActivateChopper);
			else
				g_currentMission:addHelpButtonText(g_i18n:getText("BizonGigant_ActivateChopper"), InputBinding.BizonGigant_ActivateChopper);
			end;
		end;
	end;
	
	if self.attacherVehicle ~= nil then
		local combine = self.attacherVehicle;
		if combine.isThreshing and self.chopperActive then
			self:setAnimationTime(1, 1);
		else
			self:setAnimationTime(1, 0);
		end;
		if self.chopperActive then
			combine.strawEmitState = false;
			combine.strawEnableTime = nil;
			combine.chopperActivated = true;

			if combine.lastArea > 0 then
				if combine.chopperActivated then
					if combine.chopperEnableTime == nil then
						combine.chopperEnableTime = combine.time + combine.chopperToggleTime;
					else
						combine.chopperDisableTime = nil;
					end;
					disableChopperEmit = false;
				end;
				
				if combine.chopperEnableTime ~= nil and combine.chopperEnableTime <= combine.time then
					combine.currentChopperParticleSystem = combine.chopperParticleSystems[combine.currentGrainTankFruitType];
					if combine.currentChopperParticleSystem == nil then
						combine.currentChopperParticleSystem = combine.defaultChopperParticleSystem;
					end;
					Utils.setEmittingState(combine.currentChopperParticleSystem, true);
				end;
			end;	
		else
			if combine.lastArea > 0 then
				if self.chopperActivated == true and self.chopperActive == false then
					self:activateChopper(true);
				end;
			end;
		end;
		
		for _, part in pairs(self.threshingUVScrollParts) do
			if combine.isThreshing then
				setShaderParameter(part.node, "uvScrollSpeed", part.speed[1], part.speed[2], 0, 0, false);
			else
				setShaderParameter(part.node, "uvScrollSpeed", 0, 0, 0, 0, false);
			end;
		end;
	end;
end;

function BuyableStrawChopper:updateTick(dt)   
end;

function BuyableStrawChopper:draw()
end;

function BuyableStrawChopper:onAttach(attacherVehicle)
	self.av = attacherVehicle;
	self.collisionMaskBackup = getCollisionMask(self.components[1].node);
	setCollisionMask(self.components[1].node, 1073741824);
	for _, object in pairs(self.combineExtraObjects) do
		local objectNode = Utils.indexToObject(attacherVehicle.components, object.index);
		if objectNode ~= nil then
			setVisibility(objectNode, object.showOnAttach);
		end;
	end;
	self.chopperParticleSystemsBackup = attacherVehicle.chopperParticleSystems;
end;

function BuyableStrawChopper:onDetach()
	self:activateChopper(false);
	setCollisionMask(self.components[1].node, self.collisionMaskBackup);
	self.av.chopperParticleSystems = self.chopperParticleSystemsBackup;
	for _, object in pairs(self.combineExtraObjects) do
		local objectNode = Utils.indexToObject(self.av.components, object.index);
		if objectNode ~= nil then
			setVisibility(objectNode, not object.showOnAttach);
		end;
	end;
	for _, part in pairs(self.threshingUVScrollParts) do
		setShaderParameter(part.node, "uvScrollSpeed", 0, 0, 0, 0, false);
	end;
	self:setAnimationTime(1, 0);
	self.av = {};
	self.chopperParticleSystemsBackup = {};
end;

function BuyableStrawChopper:objectInRange(object)
	if g_currentMission.player ~= nil and object ~= nil then
		local nearestDistance = 2;
		local vx, vy, vz = getWorldTranslation(g_currentMission.player.rootNode);
		local px, py, pz = getWorldTranslation(object); 
		local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);	
		if distance < nearestDistance then
			return true;
		else
			return false;
		end;
	else
		return false;
	end;
end;

function BuyableStrawChopper:activateChopper(isActive, noEventSend)
	if isActive ~= self.chopperActive then
		ActivateChopperEvent.sendEvent(self, isActive, noEventSend);
		self.chopperActive = isActive;
		if isActive then
			self:setAnimationTime(2, self.animationParts[2].animDuration);
		else
			self:setAnimationTime(2, self.animationParts[2].startPosition);
		end;
		if self.attacherVehicle ~= nil then
			if isActive then
				self.attacherVehicle.chopperParticleSystems = self.chopperParticleSystems;
			else
				self.attacherVehicle.chopperParticleSystems = self.chopperParticleSystemsBackup;
			end;
		end;
	end;
end;

function BuyableStrawChopper:onLeave()
	if self.deactivateOnLeave then
		self.onDeactivate(self);
	end;
end;

ActivateChopperEvent = {};
ActivateChopperEvent_mt = Class(ActivateChopperEvent, Event);

InitEventClass(ActivateChopperEvent, "ActivateChopperEvent");

function ActivateChopperEvent:emptyNew()
    local self = Event:new(ActivateChopperEvent_mt);
    self.className="ActivateChopperEvent";
    return self;
end;

function ActivateChopperEvent:new(vehicle, chopperState)
    local self = ActivateChopperEvent:emptyNew()
    self.vehicle = vehicle;
	self.chopperState = chopperState;
    return self;
end;

function ActivateChopperEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
	self.chopperState = streamReadBool(streamId);
    self.vehicle = networkGetObject(id);
    self:run(connection);
end;

function ActivateChopperEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteBool(streamId, self.chopperState);
end;

function ActivateChopperEvent:run(connection)   
	self.vehicle:activateChopper(self.chopperState, true);
    if not connection:getIsServer() then
        g_server:broadcastEvent(ActivateChopperEvent:new(self.vehicle, self.chopperState), nil, connection, self.vehicle);
    end;
end;

function ActivateChopperEvent.sendEvent(vehicle, chopperState, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(ActivateChopperEvent:new(vehicle, chopperState), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(ActivateChopperEvent:new(vehicle, chopperState));
		end;
	end;
end;